@@ -46,6 +46,7 @@
 #include "src/models/sort_filter_proxy_model.h"
 #include "src/settings/preferences.h"
 #include "src/single/single_instance.h"
+#include "src/web/json.h"
 #include "src/worker/task.h" /* TODO -- remove this header file. */
 
 /* Forward declaration as we don;t wan to pull-in all header file content. */
@@ -89,6 +90,30 @@ public:
 	static
 	MessageDbSet *accountDbSet(const QString &userName, MainWindow *mw);
 
+	/*!
+	 * @brief Get data about logged in user and his box.
+	 */
+	static
+	bool getOwnerInfoFromLogin(const QString &userName);
+
+	/*!
+	 * @brief Get information about password expiration date.
+	 */
+	static
+	bool getPasswordInfoFromLogin(const QString &userName);
+
+	/*!
+	 * @brief Get data about logged in user.
+	 */
+	static
+	bool getUserInfoFromLogin(const QString &userName);
+
+	/*!
+	 * @brief Get account list from Webdatovka.
+	 */
+	bool wdGetAccountList(const QString &userName,
+	    const QNetworkCookie &sessionid, bool syncWithAll);
+
 protected:
 	/*!
 	 * Check if some worker is working on the background and show
@@ -97,7 +122,18 @@ protected:
 	void closeEvent(QCloseEvent *event);
 
 
+public slots:
+
+	/*!
+	 * @brief Login to mojeID.
+	 */
+	void loginToMojeId(const QString &userName);
+
 private slots:
+	/*!
+	 * @brief Refresh AccountList.
+	 */
+	void refreshAccountList(const QString &userName);
 
 	/*!
 	 * @brief Processes messages from single instance emitter.
@@ -117,6 +153,14 @@ private slots:
 	    const QDateTime &deliveryTime, int result, const QString &errDesc,
 	    bool listScheduled);
 
+	/*!
+	 * @brief Performs action depending on message download outcome
+	 *        for webdatovka.
+	 */
+	void collectDownloadMessageMojeId(const QString &usrName,
+	    qint64 msgId, int result, const QString &errDesc,
+	    bool listScheduled);
+
 	/*!
 	 * @brief Performs action depending on message list download outcome.
 	 */
@@ -132,6 +176,12 @@ private slots:
 	    const QString &dbIDRecipient, const QString &recipientName,
 	    bool isPDZ, qint64 dmId);
 
+	/*!
+	 * @brief Performs action depending on webdatovka message send outcome.
+	 */
+	void sendMessageMojeIdAction(const QString &userName,
+	    const QStringList &result, const QString &error);
+
 	/*!
 	 * @brief Version response slot.
 	 */
@@ -223,7 +273,7 @@ private slots:
 	/*!
 	 * @brief Open attachment in default application.
 	 */
-	void openSelectedAttachment(void);
+	void openSelectedAttachment(const QModelIndex &index = QModelIndex());
 
 	/*!
 	 * @brief Mark all received messages in the current working account.
@@ -340,6 +390,11 @@ private slots:
 	 */
 	void deleteMessage(void);
 
+	/*!
+	 * @brief Delete selected message(s) from local database and Webdatovka.
+	 */
+	void deleteMessageWebdatovka(const QString &userName);
+
 	/*!
 	 * @brief Downloads new messages from server for all accounts.
 	 */
@@ -370,6 +425,12 @@ private slots:
 	 */
 	void createAndSendMessageReply(void);
 
+	/*!
+	 * @nrief Create a message containing as attachment the selected
+	 *     messages on ZFO format.
+	 */
+	void createAndSendMessageWithZfos(void);
+
 	/*!
 	 * @brief Create message from the template (selected message).
 	 */
@@ -400,11 +461,21 @@ private slots:
 	 */
 	void addNewAccount(void);
 
+	/*!
+	 * @brief Add mojeID account action and dialog.
+	 */
+	void addNewMojeIDAccount(void);
+
 	/*!
 	 * @brief Deletion confirmation dialog.
 	 */
 	void deleteSelectedAccount(void);
 
+	/*!
+	 * @brief Delete account.
+	 */
+	void deleteAccount(const QString &userName);
+
 	/*!
 	 * @brief Show import database directory dialog.
 	 */
@@ -421,7 +492,7 @@ private slots:
 	void prepareMsgsImportFromDatabase(void);
 
 	/*!
-	 * @brief Proxy setting dialog.
+	 * @brief Proxy settings dialogue.
 	 */
 	void proxySettings(void);
 
@@ -676,13 +747,48 @@ private slots:
 	 */
 	void vacuumMsgDbSlot(void);
 
+	void callMojeId(const QString &user, const QString &lastUrl,
+	    const QString &token, QString userName, QString pwd, QString otp,
+	    bool syncALL, const QString &certPath);
+
 private:
 
 	QTimer m_timerSyncAccounts;
 	int m_timeoutSyncAccounts;
 
+	/*!
+	 * @brief Shows tag editing dialogue.
+	 *
+	 * @no If no message identifiers are supplied then just the tags are
+	 *     edited.
+	 *
+	 * @param[in] userName Account login string. Must be supplied when
+	 *                     message identifiers are passed.
+	 * @param[in] msgIdList Messages whose tags should be edited.
+	 * @param[in] msgIdWebDatovkaList Webdatovka messages whose tags should
+	 *                                be edited.
+	 */
+	void modifyTags(const QString &userName, QList<qint64> msgIdList,
+	    QList<qint64> msgIdWebDatovkaList);
+
+	/*!
+	 * @brief Get message list from webdatovka server.
+	 */
+	bool wdGetMessageList(const QString &userName);
+
+	/*!
+	 * @brief Do sync of account in the webdatovka server.
+	 */
+	bool wdSyncAccount(const QString &userName);
+
+	/*!
+	 * @brief Show status bar text with timeout.
+	 */
 	void showStatusTextWithTimeout(const QString &qStr);
 
+	/*!
+	 * @brief Show status bar text permanently.
+	 */
 	void showStatusTextPermanently(const QString &qStr);
 
 	/*!
@@ -703,15 +809,14 @@ private:
 	 * @brief Export message into ZFO file dialogue.
 	 */
 	void exportMessageAsZFO(const QString &attachPath,
-	    const QString &userName, qint64 dmId, QDateTime deliveryTime,
-	    bool askLocation);
+	    const QString &userName, MessageDb::MsgId msgId, bool askLocation);
 
 	/*!
 	 * @brief Export delivery information as ZFO file dialogue.
 	 */
 	void exportDeliveryInfoAsZFO(const QString &attachPath,
 	    const QString &attachFileName, const QString &formatString,
-	    const QString &userName, qint64 dmId, QDateTime deliveryTime,
+	    const QString &userName, MessageDb::MsgId msgId,
 	    bool askLocation);
 
 	/*!
@@ -719,22 +824,19 @@ private:
 	 */
 	void exportDeliveryInfoAsPDF(const QString &attachPath,
 	    const QString &attachFileName, const QString &formatString,
-	    const QString &userName, qint64 dmId, QDateTime deliveryTime,
-	    bool askLocation);
+	    const QString &userName, MessageDb::MsgId msgId, bool askLocation);
 
 	/*!
 	 * @brief Export selected message envelope as PDF file dialogue.
 	 */
 	void exportMessageEnvelopeAsPDF(const QString &attachPath,
-	    const QString &userName, qint64 dmId, QDateTime deliveryTime,
-	    bool askLocation);
+	    const QString &userName, MessageDb::MsgId msgId, bool askLocation);
 
 	/*!
 	 * @brief Export selected message envelope as PDF and attachment files.
 	 */
 	void exportMessageEnvelopeAttachments(const QString &attachPath,
-	    const QString &userName, qint64 dmId, QDateTime deliveryTime,
-	    bool askLocation);
+	    const QString &userName, MessageDb::MsgId msgId, bool askLocation);
 
 	/*!
 	 * @brief Set info status bar from worker.
@@ -742,22 +844,23 @@ private:
 	void dataFromWorkerToStatusBarInfo(bool add,
 	    int rt, int rn, int st, int sn);
 
-	/*!
-	 * @brief Refresh AccountList.
-	 */
-	void refreshAccountList(const QString &userName);
-
 	/*!
 	 * @brief Set tablewidget when message download worker is done.
 	 */
 	void postDownloadSelectedMessageAttachments(const QString &userName,
 	    qint64 dmId);
 
+	/*!
+	 * @brief Generates file path where sto store attachment into.
+	 */
+	QString attachmentFilePath(const QString &userName,
+	    const MessageDb::MsgId &msgId, QModelIndex attIdx);
+
 	/*!
 	 * @brief Save attachment identified by indexes to file.
 	 */
-	void saveAttachmentToFile(const QModelIndex &messageIndex,
-	    const QModelIndex &attachmentIndex);
+	void saveAttachmentToFile(const QString &userName,
+	    const MessageDb::MsgId &msgId, const QModelIndex &attIdx);
 
 	/*!
 	 * @brief Return index for yearly entry with given properties.
@@ -848,12 +951,12 @@ private:
 	void activeAccountMenuAndButtons(bool action) const;
 
 	/*!
-	 * @brief Load and apply setting from configuration file.
+	 * @brief Load and apply settings from configuration file.
 	 */
 	void loadSettings(void);
 
 	/*!
-	 * @brief Store current setting to configuration file.
+	 * @brief Store current settings to configuration file.
 	 */
 	void saveSettings(void) const;
 
@@ -990,8 +1093,8 @@ private:
 	 *
 	 * @return True if message has been removed from local database.
 	 */
-	bool eraseMessage(const QString &userName, qint64 dmId,
-	     const QDateTime &deliveryTime, bool delFromIsds);
+	bool eraseMessage(const QString &userName,
+	    const MessageDb::MsgId &msgId, bool delFromIsds);
 
 	/*!
 	 * @brief Authenticate message from ZFO file.
@@ -1028,9 +1131,11 @@ private:
 	 * without worker and thread.
 	 *
 	 * @note Delivery time may change if invalid given.
+	 *
+	 * @param[in,out] msgId Message identifier.
+	 * @return True on success.
 	 */
-	bool downloadCompleteMessage(qint64 dmId,
-	    QDateTime &deliveryTime);
+	bool downloadCompleteMessage(MessageDb::MsgId &msgId);
 
 	/*!
 	 * @brief Shows notification dialogue and offers downloading of
@@ -1038,10 +1143,11 @@ private:
 	 *
 	 * @note Delivery time may change if invalid given.
 	 *
+	 * @param[in,out] msgId Message identifier.
 	 * @return True on success.
 	 */
-	bool messageMissingOfferDownload(qint64 dmId,
-	    QDateTime &deliveryTime, const QString &title);
+	bool messageMissingOfferDownload(MessageDb::MsgId &msgId,
+	    const QString &title);
 
 	/*!
 	 * @brief Set read status to messages with given indexes.
@@ -1115,6 +1221,22 @@ private:
 	bool setBackOriginDb(MessageDbSet *dbset, const QString &dbDir,
 	    const QString &userName);
 
+	/*!
+	 * @brief Show dialog for webdatovka account
+	 *        that action is not implemented.
+	 * @param[in] userName - account username.
+	 * @param[in] txt        additional info text.
+	 */
+	void showWebDatovkaInfoDialog(const QString &userName, QString txt);
+
+	/*!
+	 * @brief Test if exists another mojeID account with same userId
+	 *        when we delete any mojeID acccount.
+	 *
+	 * @param[in] userName - account username.
+	 * @return true if any account exists
+	 */
+	bool existsAnotherMojeIdAccountWithSameUserId(const QString &userName);
 
 	QString m_confDirName; /*!< Configuration directory location. */
 	QString m_confFileName; /*!< Configuration file location. */
@@ -1184,7 +1306,6 @@ private:
 	 * @brief Sets action icons.
 	 */
 	void setMenuActionIcons(void);
-
 };
 
 
