@@ -21,8 +21,6 @@
  * the two.
  */
 
-
-#include <QDesktopServices>
 #include <QInputDialog>
 #include <QMessageBox>
 #include <QDir>
@@ -33,23 +31,48 @@
 #include "src/gui/dlg_change_pwd.h"
 #include "src/gui/dlg_contacts.h"
 #include "src/gui/dlg_ds_search.h"
+#include "src/model_interaction/attachment_interaction.h"
+#include "src/gui/dlg_search_mojeid.h"
 #include "src/models/accounts_model.h"
 #include "src/io/account_db.h"
 #include "src/io/dbs.h"
-#include "src/io/filesystem.h"
 #include "src/io/isds_sessions.h"
+#include "src/io/wd_sessions.h"
 #include "src/io/message_db.h"
 #include "src/log/log.h"
 #include "src/settings/preferences.h"
-#include "src/views/attachment_table_widget.h"
+#include "src/views/attachment_table_view.h"
 #include "src/views/table_home_end_filter.h"
 #include "src/worker/message_emitter.h"
 #include "src/worker/pool.h"
 #include "src/worker/task_download_credit_info.h"
 #include "src/worker/task_keep_alive.h"
 #include "src/worker/task_send_message.h"
+#include "src/worker/task_send_message_mojeid.h"
 #include "ui_dlg_send_message.h"
 
+const QString &dzPrefix(MessageDb *messageDb, qint64 dmId)
+{
+	const static QString nothing;
+	const static QString received(QLatin1String("D"));
+	const static QString sent(QLatin1String("0"));
+
+	if (0 == messageDb || dmId < 0) {
+		return nothing;
+	}
+
+	switch (messageDb->msgMessageType(dmId)) {
+	case MessageDb::TYPE_RECEIVED:
+		return received;
+		break;
+	case MessageDb::TYPE_SENT:
+		return sent;
+		break;
+	default:
+		return nothing;
+		break;
+	}
+}
 
 /*
  * Column indexes into recipient table widget.
@@ -62,13 +85,12 @@
 
 DlgSendMessage::DlgSendMessage(
     const QList<Task::AccountDescr> &messageDbSetList,
-    Action action, qint64 msgId, const QDateTime &deliveryTime,
+    Action action, const QList<MessageDb::MsgId> &msgIds,
     const QString &userName, class MainWindow *mv, QWidget *parent)
     : QDialog(parent),
     m_keepAliveTimer(),
     m_messageDbSetList(messageDbSetList),
-    m_msgID(msgId),
-    m_deliveryTime(deliveryTime),
+    m_msgIds(msgIds),
     m_dbId(""),
     m_senderName(""),
     m_action(action),
@@ -81,12 +103,21 @@ DlgSendMessage::DlgSendMessage(
     m_dmType(""),
     m_dmSenderRefNumber(""),
     m_mv(mv),
+    m_dbSet(0),
     m_isLogged(false),
+    m_attachmentModel(),
+    m_isWebDatovkaAccount(false),
     m_transactIds(),
     m_sentMsgResultList()
 {
 	setupUi(this);
+	/* Set default line height for table views/widgets. */
+	recipientTableWidget->setNarrowedLineHeight();
+	attachmentTableView->setNarrowedLineHeight();
+
 	initNewMessageDialog();
+
+	Q_ASSERT(0 != m_dbSet);
 }
 
 /* ========================================================================= */
@@ -104,15 +135,47 @@ void DlgSendMessage::on_cancelButton_clicked(void)
 void DlgSendMessage::initNewMessageDialog(void)
 /* ========================================================================= */
 {
+	if (isWebDatovkaAccount(m_userName)) {
+		m_isWebDatovkaAccount = true;
+	}
+
 	this->recipientTableWidget->setColumnWidth(0,70);
 	this->recipientTableWidget->setColumnWidth(1,180);
 	this->recipientTableWidget->setColumnWidth(2,240);
 
-	this->attachmentTableWidget->setColumnWidth(0,150);
-	this->attachmentTableWidget->setColumnWidth(1,40);
-	this->attachmentTableWidget->setColumnWidth(2,120);
+	m_attachmentModel.setHeader();
 
-	this->attachmentTableWidget->setColumnHidden(5, true);
+	this->attachmentTableView->setModel(&m_attachmentModel);
+
+	this->attachmentTableView->setColumnWidth(DbFlsTblModel::FNAME_COL, 150);
+	this->attachmentTableView->setColumnWidth(DbFlsTblModel::MIME_COL, 120);
+
+	this->attachmentTableView->setColumnHidden(DbFlsTblModel::ATTACHID_COL, true);
+	this->attachmentTableView->setColumnHidden(DbFlsTblModel::MSGID_COL, true);
+	this->attachmentTableView->setColumnHidden(DbFlsTblModel::CONTENT_COL, true);
+
+#if 0
+	this->attachmentTableView->setAcceptDrops(true);
+	this->attachmentTableView->setDragEnabled(false);
+	this->attachmentTableView->setDragDropOverwriteMode(false);
+	this->attachmentTableView->setDropIndicatorShown(true);
+	this->attachmentTableView->setDragDropMode(QAbstractItemView::InternalMove);
+//	this->attachmentTableView->setDefaultDropAction(Qt::MoveAction);
+#endif
+#if 0
+	this->attachmentTableView->setAcceptDrops(true);
+	this->attachmentTableView->setDragEnabled(true);
+	this->attachmentTableView->setDragDropOverwriteMode(false);
+	this->attachmentTableView->setDropIndicatorShown(true);
+	this->attachmentTableView->setDragDropMode(QAbstractItemView::DragDrop);
+	this->attachmentTableView->setDefaultDropAction(Qt::MoveAction);
+#endif
+	this->attachmentTableView->setAcceptDrops(true);
+	this->attachmentTableView->setDragEnabled(true);
+	this->attachmentTableView->setDragDropOverwriteMode(false);
+	this->attachmentTableView->setDropIndicatorShown(true);
+	this->attachmentTableView->setDragDropMode(QAbstractItemView::DragDrop);
+	this->attachmentTableView->setDefaultDropAction(Qt::CopyAction);
 
 	this->replyLabel->hide();
 	this->replyLabel->setEnabled(false);
@@ -162,15 +225,14 @@ void DlgSendMessage::initNewMessageDialog(void)
 	connect(this->removeAttachment, SIGNAL(clicked()), this,
 	    SLOT(deleteSelectedAttachmentFiles()));
 	connect(this->openAttachment, SIGNAL(clicked()), this,
-	    SLOT(openAttachmentFile()));
+	    SLOT(openSelectedAttachment()));
 
 	connect(this->recipientTableWidget,
 	    SIGNAL(itemClicked(QTableWidgetItem *)), this,
 	    SLOT(recItemSelect()));
 
-	connect(this->attachmentTableWidget,
-	    SIGNAL(itemDoubleClicked(QTableWidgetItem *)),
-	    this, SLOT(tableItemDoubleClicked(QTableWidgetItem *)));
+	connect(this->attachmentTableView, SIGNAL(doubleClicked(QModelIndex)),
+	    this, SLOT(openSelectedAttachment(QModelIndex)));
 
 	connect(this->enterDbIdpushButton, SIGNAL(clicked()), this,
 	    SLOT(addDbIdToRecipientList()));
@@ -178,27 +240,27 @@ void DlgSendMessage::initNewMessageDialog(void)
 	connect(this->subjectText, SIGNAL(textChanged(QString)),
 	    this, SLOT(checkInputFields()));
 
-	connect(this->attachmentTableWidget->model(),
+	connect(&m_attachmentModel,
 	    SIGNAL(rowsInserted(QModelIndex, int, int)), this,
 	    SLOT(checkInputFields()));
-	connect(this->attachmentTableWidget->model(),
+	connect(&m_attachmentModel,
 	    SIGNAL(rowsRemoved(QModelIndex, int, int)), this,
 	    SLOT(checkInputFields()));
-	connect(this->attachmentTableWidget->model(),
+	connect(&m_attachmentModel,
 	    SIGNAL(dataChanged(QModelIndex, QModelIndex, QVector<int>)), this,
 	    SLOT(attachmentDataChanged(QModelIndex, QModelIndex, QVector<int>)));
-	connect(this->attachmentTableWidget->selectionModel(),
+	connect(this->attachmentTableView->selectionModel(),
 	    SIGNAL(selectionChanged(QItemSelection, QItemSelection)), this,
 	    SLOT(attachmentSelectionChanged(QItemSelection, QItemSelection)));
 
 	this->recipientTableWidget->
 	    setEditTriggers(QAbstractItemView::NoEditTriggers);
-	this->attachmentTableWidget->
+	this->attachmentTableView->
 	    setEditTriggers(QAbstractItemView::NoEditTriggers);
 
 	this->recipientTableWidget->installEventFilter(
 	    new TableHomeEndFilter(this));
-	this->attachmentTableWidget->installEventFilter(
+	this->attachmentTableView->installEventFilter(
 	    new TableHomeEndFilter(this));
 
 	connect(this->sendButton, SIGNAL(clicked()), this, SLOT(sendMessage()));
@@ -207,11 +269,17 @@ void DlgSendMessage::initNewMessageDialog(void)
 	        QString, QString, bool, qint64)), this,
 	    SLOT(collectSendMessageStatus(QString, QString, int, QString,
 	        QString, QString, bool, qint64)));
+	connect(&globMsgProcEmitter,
+	    SIGNAL(sendMessageMojeIdFinished(QString, QStringList, QString)), this,
+	    SLOT(sendMessageMojeIdAction(QString, QStringList,  QString)));
 
-	connect(&m_keepAliveTimer, SIGNAL(timeout()), this,
-	    SLOT(pingIsdsServer()));
 	m_keepAliveTimer.start(DLG_ISDS_KEEPALIVE_MS);
 
+	if (!m_isWebDatovkaAccount) {
+		connect(&m_keepAliveTimer, SIGNAL(timeout()), this,
+		    SLOT(pingIsdsServer()));
+	}
+
 	this->attachmentSizeInfo->setText(
 	    tr("Total size of attachments is %1 B").arg(0));
 
@@ -235,6 +303,8 @@ void DlgSendMessage::initNewMessageDialog(void)
 		this->payRecipient->hide();
 		if (ACT_NEW_FROM_TMP == m_action) {
 			fillDlgFromTmpMsg();
+		} else if (ACT_FORWARD == m_action) {
+			fillDlgAsForward();
 		}
 	}
 
@@ -265,32 +335,42 @@ void DlgSendMessage::setAccountInfo(int item)
 		m_userName = userName;
 	}
 
-	struct isds_ctx *session = NULL;
-
-	m_isLogged = true;
-
-	m_keepAliveTimer.stop();
-	{
-		TaskKeepAlive *task;
-		task = new (std::nothrow) TaskKeepAlive(m_userName);
-		task->setAutoDelete(false);
-		globWorkPool.runSingle(task);
-
-		m_isLogged = task->m_isAlive;
-
-		delete task;
+	if (isWebDatovkaAccount(m_userName)) {
+		m_isWebDatovkaAccount = true;
 	}
-	if (!m_isLogged) {
-		if (m_mv) {
-			m_isLogged = m_mv->connectToIsds(m_userName);
+
+	if (!m_isWebDatovkaAccount) {
+
+		struct isds_ctx *session = NULL;
+		m_isLogged = true;
+		m_keepAliveTimer.stop();
+		{
+			TaskKeepAlive *task;
+			task = new (std::nothrow) TaskKeepAlive(m_userName);
+			task->setAutoDelete(false);
+			globWorkPool.runSingle(task);
+
+			m_isLogged = task->m_isAlive;
+
+			delete task;
 		}
-	}
-	m_keepAliveTimer.start(DLG_ISDS_KEEPALIVE_MS);
+		if (!m_isLogged) {
+			if (m_mv) {
+				m_isLogged = m_mv->connectToIsds(m_userName);
+			}
+		}
+		m_keepAliveTimer.start(DLG_ISDS_KEEPALIVE_MS);
 
-	session = globIsdsSessions.session(m_userName);
-	if (NULL == session) {
-		logErrorNL("%s", "Missing ISDS session.");
-		m_isLogged = false;
+		session = globIsdsSessions.session(m_userName);
+		if (NULL == session) {
+			logErrorNL("%s", "Missing ISDS session.");
+			m_isLogged = false;
+		}
+	} else {
+		if (!wdSessions.isConnectedToWebdatovka(m_userName)) {
+			m_mv->loginToMojeId(m_userName);
+		}
+		m_isLogged = true;
 	}
 
 	foreach (const Task::AccountDescr &acnt, m_messageDbSetList) {
@@ -317,8 +397,11 @@ void DlgSendMessage::setAccountInfo(int item)
 	} else {
 		m_lastAttAddPath = accountInfo.lastAttachAddPath();
 	}
-	if (m_dbOpenAddressing) {
-		m_pdzCredit = getPDZCreditFromISDS(m_userName, m_dbId);
+
+	if (!m_isWebDatovkaAccount) {
+		if (m_dbOpenAddressing) {
+			m_pdzCredit = getPDZCreditFromISDS(m_userName, m_dbId);
+		}
 	}
 
 	QString dbOpenAddressingText = "";
@@ -340,7 +423,6 @@ void DlgSendMessage::setAccountInfo(int item)
 }
 
 
-
 /* ========================================================================= */
 /*
  * Func: Return remaining PDZ credit.
@@ -368,21 +450,6 @@ QString DlgSendMessage::getPDZCreditFromISDS(const QString &userName,
 }
 
 
-/* ========================================================================= */
-/*
- * Slot is fired when user double clicked on attachment item - open file.
- */
-void DlgSendMessage::tableItemDoubleClicked(QTableWidgetItem *item)
-/* ========================================================================= */
-{
-	debugSlotCall();
-
-	Q_UNUSED(item);
-
-	openAttachmentFile();
-}
-
-
 /* ========================================================================= */
 /*
  * Whenever any data in attachment table change.
@@ -415,12 +482,13 @@ void DlgSendMessage::attachmentSelectionChanged(const QItemSelection &selected,
 	QModelIndexList selectedIndexes;
 	{
 		QItemSelectionModel *selectionModel =
-		    this->attachmentTableWidget->selectionModel();
+		    this->attachmentTableView->selectionModel();
 		if (0 == selectionModel) {
 			Q_ASSERT(0);
 			return;
 		}
-		selectedIndexes = selectionModel->selectedRows(0);
+		selectedIndexes =
+		    selectionModel->selectedRows(DbFlsTblModel::FNAME_COL);
 	}
 
 	this->removeAttachment->setEnabled(selectedIndexes.size() > 0);
@@ -437,15 +505,23 @@ void DlgSendMessage::fillDlgAsReply(void)
 {
 	debugFuncCall();
 
+	if (m_msgIds.size() != 1) {
+		logWarningNL("%s",
+		    "Expected one message to generate reply from.");
+		return;
+	}
+	const MessageDb::MsgId &msgId(m_msgIds.first());
+
 	bool hideOptionalWidget = true;
 
 	this->fromComboBox->setEnabled(false);
 
-	MessageDb *messageDb = m_dbSet->accessMessageDb(m_deliveryTime, false);
+	MessageDb *messageDb =
+	    m_dbSet->accessMessageDb(msgId.deliveryTime, false);
 	Q_ASSERT(0 != messageDb);
 
 	MessageDb::PartialEnvelopeData envData =
-	    messageDb->msgsReplyData(m_msgID);
+	    messageDb->msgsReplyData(msgId.dmId);
 	m_dmType = envData.dmType;
 	m_dmSenderRefNumber = envData.dmRecipientRefNumber;
 
@@ -517,6 +593,42 @@ void DlgSendMessage::fillDlgAsReply(void)
 	this->recipientTableWidget->setItem(row, RTW_PDZ, item);
 }
 
+void DlgSendMessage::fillDlgAsForward(void)
+{
+	debugFuncCall();
+
+	if (m_msgIds.size() == 0) {
+		logWarningNL("%s",
+		    "Expected at lease one message to generate reply from.");
+		return;
+	}
+
+	/* Fill attachments with messages. */
+	foreach (const MessageDb::MsgId &msgId, m_msgIds) {
+		MessageDb *messageDb =
+		    m_dbSet->accessMessageDb(msgId.deliveryTime, false);
+		if (0 == messageDb) {
+			Q_ASSERT(0);
+			continue;
+		}
+
+		/* If only a single message if forwarded. */
+		if (m_msgIds.size() == 1) {
+			MessageDb::PartialEnvelopeData envData(
+			    messageDb->msgsReplyData(msgId.dmId));
+
+			this->subjectText->setText("Fwd: " + envData.dmAnnotation);
+		}
+
+		QByteArray msgBase64(messageDb->msgsMessageBase64(msgId.dmId));
+		if (msgBase64.isEmpty()) {
+			continue;
+		}
+
+		m_attachmentModel.appendAttachmentEntry(msgBase64,
+		    dzPrefix(messageDb, msgId.dmId) + QString("DZ_%1.zfo").arg(msgId.dmId));
+	}
+}
 
 /* ========================================================================= */
 /*
@@ -527,13 +639,21 @@ void DlgSendMessage::fillDlgFromTmpMsg(void)
 {
 	debugFuncCall();
 
+	if (m_msgIds.size() != 1) {
+		logWarningNL("%s",
+		    "Expected one message to generate reply from.");
+		return;
+	}
+	const MessageDb::MsgId &msgId(m_msgIds.first());
+
 	bool hideOptionalWidget = true;
 
-	MessageDb *messageDb = m_dbSet->accessMessageDb(m_deliveryTime, false);
+	MessageDb *messageDb =
+	    m_dbSet->accessMessageDb(msgId.deliveryTime, false);
 	Q_ASSERT(0 != messageDb);
 
 	MessageDb::PartialEnvelopeData envData =
-	    messageDb->msgsReplyData(m_msgID);
+	    messageDb->msgsReplyData(msgId.dmId);
 	m_dmType = envData.dmType;
 	m_dmSenderRefNumber = envData.dmRecipientRefNumber;
 
@@ -621,30 +741,11 @@ void DlgSendMessage::fillDlgFromTmpMsg(void)
 
 	/* fill attachments from template message */
 	QList<MessageDb::FileData> msgFileList =
-	    messageDb->getFilesFromMessage(m_msgID);
+	    messageDb->getFilesFromMessage(msgId.dmId);
 
 	foreach (const MessageDb::FileData &fileData, msgFileList) {
-		int row = this->attachmentTableWidget->rowCount();
-		this->attachmentTableWidget->insertRow(row);
-		QTableWidgetItem *item = new QTableWidgetItem;
-		item->setText(fileData.dmFileDescr);
-		this->attachmentTableWidget->setItem(row, ATW_FILE, item);
-		item = new QTableWidgetItem;
-		item->setText("");
-		this->attachmentTableWidget->setItem(row, ATW_TYPE, item);
-		item = new QTableWidgetItem;
-		item->setText(tr("unknown"));
-		this->attachmentTableWidget->setItem(row, ATW_MIME, item);
-		item = new QTableWidgetItem;
-		item->setText(QString::number(
-		    base64RealSize(fileData.dmEncodedContent)));
-		this->attachmentTableWidget->setItem(row, ATW_SIZE, item);
-		item = new QTableWidgetItem;
-		item->setText(tr("local database"));
-		this->attachmentTableWidget->setItem(row, ATW_PATH, item);
-		item = new QTableWidgetItem;
-		item->setData(Qt::DisplayRole, fileData.dmEncodedContent);
-		this->attachmentTableWidget->setItem(row, ATW_DATA, item);
+		m_attachmentModel.appendAttachmentEntry(
+		    fileData.dmEncodedContent, fileData.dmFileDescr);
 	}
 }
 
@@ -686,8 +787,10 @@ void DlgSendMessage::addAttachmentFile(void)
 	}
 
 	foreach (const QString &fileName, fileNames) {
-		int fileSize = this->attachmentTableWidget->addFile(fileName);
+		int fileSize = m_attachmentModel.insertAttachmentFile(fileName,
+		    m_attachmentModel.rowCount());
 		if (fileSize <= 0) {
+			/* TODO -- Generate some warning message. */
 			continue;
 		}
 	}
@@ -772,7 +875,7 @@ void DlgSendMessage::deleteSelectedAttachmentFiles(void)
 /* ========================================================================= */
 {
 	QModelIndexList firstMsgColumnIdxs =
-	   this->attachmentTableWidget->selectionModel()->selectedRows(0);
+	   this->attachmentTableView->selectionModel()->selectedRows(0);
 
 	for (int i = firstMsgColumnIdxs.size() - 1; i >= 0; --i) {
 		/*
@@ -780,7 +883,7 @@ void DlgSendMessage::deleteSelectedAttachmentFiles(void)
 		 * indexes.
 		 */
 		int row = firstMsgColumnIdxs.at(i).row();
-		this->attachmentTableWidget->removeRow(row);
+		m_attachmentModel.removeRow(row);
 	}
 
 	checkInputFields();
@@ -796,22 +899,23 @@ bool DlgSendMessage::calculateAndShowTotalAttachSize(void)
 {
 	int aSize = 0;
 
-	for (int i = 0; i < this->attachmentTableWidget->rowCount(); i++) {
-		QTableWidgetItem *item =
-		    this->attachmentTableWidget->item(i, ATW_SIZE);
-		if (0 != item) {
-			aSize += item->text().toInt();
+	for (int i = 0; i < m_attachmentModel.rowCount(); ++i) {
+		QModelIndex idx(
+		    m_attachmentModel.index(i, DbFlsTblModel::FSIZE_COL));
+
+		if (idx.isValid()) {
+			aSize += idx.data().toInt();
 		}
 	}
 
 	this->attachmentSizeInfo->setStyleSheet("QLabel { color: black }");
 
-	if (this->attachmentTableWidget->rowCount() > MAX_ATTACHMENT_FILES) {
+	if (m_attachmentModel.rowCount() > MAX_ATTACHMENT_FILES) {
 		this->attachmentSizeInfo->
 		     setStyleSheet("QLabel { color: red }");
-		this->attachmentSizeInfo->setText(
-		    tr("Warning: The permitted amount (%1) of attachments has "
-		    "been exceeded.").arg(QString::number(MAX_ATTACHMENT_FILES)));
+		this->attachmentSizeInfo->setText(tr(
+		    "Warning: The permitted amount (%1) of attachments has been exceeded.")
+		        .arg(QString::number(MAX_ATTACHMENT_FILES)));
 		return false;
 	}
 
@@ -825,15 +929,14 @@ bool DlgSendMessage::calculateAndShowTotalAttachSize(void)
 				this->attachmentSizeInfo->
 				     setStyleSheet("QLabel { color: red }");
 				this->attachmentSizeInfo->setText(
-				    tr("Warning: Total size of attachments "
-				        "is larger than %1 MB!").
-				    arg(QString::number(
+				    tr("Warning: Total size of attachments is larger than %1 MB!")
+				    .arg(QString::number(
 				        MAX_ATTACHMENT_SIZE_MB)));
 				return false;
 			}
 		} else {
-			this->attachmentSizeInfo->setText(
-			   tr("Total size of attachments is ~%1 B").arg(aSize));
+			this->attachmentSizeInfo->setText(tr(
+			    "Total size of attachments is ~%1 B").arg(aSize));
 		}
 	} else {
 		this->attachmentSizeInfo->setText(
@@ -852,9 +955,9 @@ void DlgSendMessage::checkInputFields(void)
 /* ========================================================================= */
 {
 	bool buttonEnabled = calculateAndShowTotalAttachSize()
-		    && !this->subjectText->text().isEmpty()
-		    && (this->recipientTableWidget->rowCount() > 0)
-		    && (this->attachmentTableWidget->rowCount() > 0);
+	    && !this->subjectText->text().isEmpty()
+	    && (this->recipientTableWidget->rowCount() > 0)
+	    && (m_attachmentModel.rowCount() > 0);
 
 	if (this->payReply->isChecked()) {
 		if (this->dmSenderRefNumber->text().isEmpty()) {
@@ -887,70 +990,30 @@ void DlgSendMessage::deleteRecipientData(void)
 
 /* ========================================================================= */
 /*
- * Find recipent in the ISDS.
+ * Find recipient in the ISDS.
  */
 void DlgSendMessage::findAndAddRecipient(void)
 /* ========================================================================= */
 {
-	QDialog *dsSearch = new DlgDsSearch(DlgDsSearch::ACT_ADDNEW,
-	    this->recipientTableWidget, m_dbType, m_dbEffectiveOVM,
-	    m_dbOpenAddressing, this, m_userName);
+	QDialog *dsSearch = NULL;
+	if (!m_isWebDatovkaAccount) {
+		dsSearch = new DlgDsSearch(DlgDsSearch::ACT_ADDNEW,
+		    this->recipientTableWidget, m_dbType, m_dbEffectiveOVM,
+		    m_dbOpenAddressing, this, m_userName);
+	} else {
+		dsSearch = new DlgDsSearchMojeId(DlgDsSearchMojeId::ACT_ADDNEW,
+		    this->recipientTableWidget, m_dbType, m_dbEffectiveOVM,
+		    this, m_userName);
+	}
 	dsSearch->exec();
 }
 
-
-/* ========================================================================= */
-/*
- * Open attachment in default application.
- */
-void DlgSendMessage::openAttachmentFile(void)
-/* ========================================================================= */
+void DlgSendMessage::openSelectedAttachment(const QModelIndex &index)
 {
-	QModelIndex selectedIndex =
-	    this->attachmentTableWidget->currentIndex();
+	debugSlotCall();
 
-	Q_ASSERT(selectedIndex.isValid());
-	if (!selectedIndex.isValid()) {
-		return;
-	}
-
-	QModelIndex fileNameIndex =
-	    selectedIndex.sibling(selectedIndex.row(), ATW_FILE);
-	Q_ASSERT(fileNameIndex.isValid());
-	if(!fileNameIndex.isValid()) {
-		return;
-	}
-	QString attachName = fileNameIndex.data().toString();
-	Q_ASSERT(!attachName.isEmpty());
-	if (attachName.isEmpty()) {
-		return;
-	}
-	attachName.replace(QRegExp("\\s"), "_").replace(
-	    QRegExp("[^a-zA-Z\\d\\.\\-_]"), "x");
-	/* TODO -- Add message id into file name? */
-	QString fileName = TMP_ATTACHMENT_PREFIX + attachName;
-
-	/* Get data from base64. */
-	QModelIndex dataIndex = selectedIndex.sibling(selectedIndex.row(),
-	    ATW_DATA);
-	Q_ASSERT(dataIndex.isValid());
-	if (!dataIndex.isValid()) {
-		return;
-	}
-
-	QByteArray data =
-	    QByteArray::fromBase64(dataIndex.data().toByteArray());
-
-	fileName = writeTemporaryFile(fileName, data);
-	if (!fileName.isEmpty()) {
-		QDesktopServices::openUrl(QUrl::fromLocalFile(fileName));
-		/* TODO -- Handle openUrl() return value. */
-	} else {
-		QMessageBox::warning(this,
-		    tr("Error opening attachment."),
-		    tr("Cannot write file '%1'.").arg(fileName),
-		    QMessageBox::Ok);
-	}
+	AttachmentInteraction::openAttachment(this, *this->attachmentTableView,
+	    index);
 }
 
 
@@ -995,13 +1058,18 @@ int DlgSendMessage::showInfoAboutPDZ(int pdzCnt)
 bool DlgSendMessage::buildDocuments(QList<IsdsDocument> &documents) const
 {
 	/* Load attachments. */
-	for (int i = 0; i < this->attachmentTableWidget->rowCount(); ++i) {
+	for (int row = 0; row < m_attachmentModel.rowCount(); ++row) {
 		IsdsDocument document;
+		QModelIndex index;
 
 		document.isXml = false;
 
-		document.dmFileDescr = this->attachmentTableWidget->
-		    item(i, ATW_FILE)->text();
+		index = m_attachmentModel.index(row, DbFlsTblModel::FNAME_COL);
+		if (!index.isValid()) {
+			Q_ASSERT(0);
+			continue;
+		}
+		document.dmFileDescr = index.data().toString();
 
 		/*
 		 * First document must have dmFileMetaType set to
@@ -1014,11 +1082,16 @@ bool DlgSendMessage::buildDocuments(QList<IsdsDocument> &documents) const
 		 * be filled up on the ISDS server. It allows sending files
 		 * with special mime types without recognition by application.
 		 */
-		document.dmMimeType = "";
+		document.dmMimeType = QStringLiteral("");
 
+		index =
+		    m_attachmentModel.index(row, DbFlsTblModel::CONTENT_COL);
+		if (!index.isValid()) {
+			Q_ASSERT(0);
+			continue;
+		}
 		document.data = QByteArray::fromBase64(
-		    this->attachmentTableWidget->item(i, ATW_DATA)->
-		         data(Qt::DisplayRole).toByteArray());
+		    index.data(Qt::DisplayRole).toByteArray());
 
 		documents.append(document);
 	}
@@ -1089,6 +1162,74 @@ bool DlgSendMessage::buildEnvelope(IsdsEnvelope &envelope) const
 	return true;
 }
 
+
+bool DlgSendMessage::buildEnvelopeWebDatovka(JsonLayer::Envelope &envelope) const
+{
+	/* Set mandatory fields of envelope. */
+	envelope.dmAnnotation = this->subjectText->text();
+
+	/* Set optional fields. */
+	envelope.dmSenderIdent = this->dmSenderIdent->text();
+	envelope.dmRecipientIdent = this->dmRecipientIdent->text();
+	envelope.dmSenderRefNumber = this->dmSenderRefNumber->text();
+	envelope.dmRecipientRefNumber = this->dmRecipientRefNumber->text();
+	envelope.dmLegalTitleLaw = this->dmLegalTitleLaw->text();
+	envelope.dmLegalTitleYear = this->dmLegalTitleYear->text();
+	envelope.dmLegalTitleSect = this->dmLegalTitleSect->text();
+	envelope.dmLegalTitlePar = this->dmLegalTitlePar->text();
+	envelope.dmLegalTitlePoint = this->dmLegalTitlePoint->text();
+	envelope.dmPersonalDelivery = this->dmPersonalDelivery->isChecked();
+	envelope.dmPublishOwnID = this->dmPublishOwnID->isChecked();
+	envelope.dmOVM = m_dbEffectiveOVM;
+
+	/* Only OVM can change. */
+	if (convertDbTypeToInt(m_dbType) > DBTYPE_OVM_REQ) {
+		envelope.dmAllowSubstDelivery = true;
+	} else {
+		envelope.dmAllowSubstDelivery =
+		    this->dmAllowSubstDelivery->isChecked();
+	}
+
+	return true;
+}
+
+
+/* ========================================================================= */
+/*
+ * Load attachments into json for sending via webdatovka.
+ */
+bool DlgSendMessage::buildFileListWebDatovka(QList<JsonLayer::File> &fileList)
+   const
+/* ========================================================================= */
+{
+	QModelIndex index;
+
+	for (int row = 0; row < m_attachmentModel.rowCount(); ++row) {
+
+		index = m_attachmentModel.index(row, DbFlsTblModel::FNAME_COL);
+		if (!index.isValid()) {
+			Q_ASSERT(0);
+			continue;
+		}
+
+		JsonLayer::File file;
+		file.fName = index.data().toString();
+
+		index =
+		    m_attachmentModel.index(row, DbFlsTblModel::CONTENT_COL);
+		if (!index.isValid()) {
+			Q_ASSERT(0);
+			continue;
+		}
+
+		file.fContent = index.data(Qt::DisplayRole).toByteArray();
+		fileList.append(file);
+	}
+
+	return true;
+}
+
+
 /* ========================================================================= */
 /*
  * Send message/multiple message.
@@ -1098,6 +1239,39 @@ void DlgSendMessage::sendMessage(void)
 {
 	debugSlotCall();
 
+	if (m_isWebDatovkaAccount) {
+
+		/* Get account ID */
+		int accountID = getWebDatovkaAccountId(m_userName);
+
+		/* Create recipient list. */
+		JsonLayer::Recipient recipient;
+		QList<JsonLayer::Recipient> recipientList;
+		for (int row = 0; row < this->recipientTableWidget->rowCount(); ++row) {
+			recipient.recipientDbId =
+			    this->recipientTableWidget->item(row, RTW_ID)->text();
+			recipient.toHands = this->dmToHands->text();
+			recipient.recipientName =
+			    this->recipientTableWidget->item(row, RTW_NAME)->text();
+			recipient.recipientAddress =
+			    this->recipientTableWidget->item(row, RTW_ADDR)->text();
+			recipientList.append(recipient);
+		}
+
+		JsonLayer::Envelope envelope;
+		buildEnvelopeWebDatovka(envelope);
+		QList<JsonLayer::File> fileList;
+		buildFileListWebDatovka(fileList);
+
+		TaskSendMessageMojeId *task;
+		task = new (std::nothrow) TaskSendMessageMojeId(m_userName, accountID,
+		    recipientList, envelope, fileList);
+		task->setAutoDelete(true);
+		globWorkPool.assignHi(task);
+
+		return;
+	}
+
 	QString detailText;
 
 	/* List of unique identifiers. */
@@ -1337,3 +1511,54 @@ void DlgSendMessage::addDbIdToRecipientList(void)
 		this->recipientTableWidget->setItem(row, RTW_PDZ, item);
 	}
 }
+
+
+/* ========================================================================= */
+/*
+ * Slot: Performs action depending on webdatovka message send outcome.
+ */
+void DlgSendMessage::sendMessageMojeIdAction(const QString &userName,
+    const QStringList &result, const QString &error)
+/* ========================================================================= */
+{
+	debugSlotCall();
+
+	Q_UNUSED(error);
+
+	QString detailText;
+
+	if (result.isEmpty()) {
+		QMessageBox msgBox;
+		msgBox.setIcon(QMessageBox::Information);
+		msgBox.setWindowTitle(tr("Message sent"));
+		msgBox.setText("<b>" +
+		    tr("Message has successfully been sent to all recipients.") +
+		    "</b>");
+		msgBox.setStandardButtons(QMessageBox::Ok);
+		msgBox.setDefaultButton(QMessageBox::Ok);
+		msgBox.exec();
+		this->accept(); /* Set return code to accepted. */
+		emit doActionAfterSentMsgSignal(userName, m_lastAttAddPath);
+	} else {
+		for (int i = 0; i < result.count(); ++i) {
+			QString msg = result.at(i);
+			detailText += msg.replace("§", ": ");
+		}
+		QMessageBox msgBox;
+		msgBox.setIcon(QMessageBox::Warning);
+		msgBox.setWindowTitle(tr("Message sending error"));
+		msgBox.setText("<b>" +
+		    tr("Message has NOT been sent to all recipients.") +
+		    "</b>");
+		detailText += "<br/><br/><b>" +
+		    tr("Do you want to close the Send message form?") + "</b>";
+		msgBox.setInformativeText(detailText);
+		msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
+		msgBox.setDefaultButton(QMessageBox::No);
+		if (msgBox.exec() == QMessageBox::Yes) {
+			this->close(); /* Set return code to closed. */
+			emit doActionAfterSentMsgSignal(userName,
+			    m_lastAttAddPath);
+		}
+	}
+}
