@@ -27,16 +27,31 @@
 #include <QDateTime>
 #include <QDialog>
 #include <QFileDialog>
+#include <QList>
 #include <QSet>
 #include <QTimer>
 #include <QTreeView>
 
 #include "src/common.h"
+#include "src/io/message_db.h"
 #include "src/io/message_db_set.h"
+#include "src/models/files_model.h"
 #include "src/worker/task.h"
 #include "src/worker/task_send_message.h"
+#include "src/web/json.h"
 #include "ui_dlg_send_message.h"
 
+/*!
+ * @brief Returns message prefix depending on whether it is sent ore received.
+ *
+ * @param[in] messageDb Message database.
+ * @param[in] dmId Message identifier number.
+ * @return Prefix.
+ *
+ * @todo Move this function somewhere else
+ *     (and clean up the code of the following dialogue.
+ */
+const QString &dzPrefix(MessageDb *messageDb, qint64 dmId);
 
 class DlgSendMessage : public QDialog, public Ui::SendMessage {
     Q_OBJECT
@@ -45,11 +60,12 @@ public:
 	enum Action {
 		ACT_NEW,
 		ACT_REPLY,
+		ACT_FORWARD,
 		ACT_NEW_FROM_TMP
 	};
 
 	DlgSendMessage(const QList<Task::AccountDescr> &messageDbSetList,
-	    Action action, qint64 msgId, const QDateTime &deliveryTime,
+	    Action action, const QList<MessageDb::MsgId> &msgIds,
 	    const QString &userName, class MainWindow *mv, QWidget *parent = 0);
 
 signals:
@@ -61,7 +77,14 @@ private slots:
 	void showOptionalFormAndSet(int);
 	void addAttachmentFile(void);
 	void deleteSelectedAttachmentFiles(void);
-	void openAttachmentFile(void);
+
+	/*
+	 * @brief Open attachment in default application.
+	 *
+	 * @param index Index identifying the line. If invalid index passed then
+	 *              selected item is taken from the selection model.
+	 */
+	void openSelectedAttachment(const QModelIndex &index = QModelIndex());
 	void addRecipientFromLocalContact(void);
 	void deleteRecipientData(void);
 	void findAndAddRecipient(void);
@@ -72,9 +95,10 @@ private slots:
 	    const QString &transactId, int result, const QString &resultDesc,
 	    const QString &dbIDRecipient, const QString &recipientName,
 	    bool isPDZ, qint64 dmId);
+	void sendMessageMojeIdAction(const QString &userName,
+	    const QStringList &result, const QString &error);
 	void pingIsdsServer(void);
 	void addDbIdToRecipientList(void);
-	void tableItemDoubleClicked(QTableWidgetItem *item);
 	void attachmentDataChanged(const QModelIndex &topLeft,
 	    const QModelIndex &bottomRight, const QVector<int> &roles);
 	void attachmentSelectionChanged(const QItemSelection &selected,
@@ -84,11 +108,10 @@ private slots:
 private:
 	QTimer m_keepAliveTimer;
 	const QList<Task::AccountDescr> m_messageDbSetList;
-	qint64 m_msgID;
-	QDateTime m_deliveryTime;
+	const QList<MessageDb::MsgId> m_msgIds;
 	QString m_dbId;
 	QString m_senderName;
-	Action m_action;
+	const Action m_action;
 	QString m_userName;
 	QString m_dbType;
 	bool m_dbEffectiveOVM;
@@ -100,6 +123,8 @@ private:
 	class MainWindow *m_mv;
 	MessageDbSet *m_dbSet;
 	bool m_isLogged;
+	DbFlsTblModel m_attachmentModel; /*!< Attachment model. */
+	bool m_isWebDatovkaAccount;
 
 	/* Used to collect sending results. */
 	QSet<QString> m_transactIds;
@@ -108,11 +133,14 @@ private:
 	void initNewMessageDialog(void);
 	bool calculateAndShowTotalAttachSize(void);
 	void fillDlgAsReply(void);
+	void fillDlgAsForward(void);
 	void fillDlgFromTmpMsg(void);
 	int showInfoAboutPDZ(int pdzCnt);
 
 	bool buildDocuments(QList<IsdsDocument> &documents) const;
 	bool buildEnvelope(IsdsEnvelope &envelope) const;
+	bool buildEnvelopeWebDatovka(JsonLayer::Envelope &envelope) const;
+	bool buildFileListWebDatovka(QList<JsonLayer::File> &fileList) const;
 
 	static
 	QString getPDZCreditFromISDS(const QString &userName,
