@@ -21,16 +21,13 @@
  * the two.
  */
 
-
 #include <QDateTime>
-#include <QDesktopServices>
 #include <QDialog>
 #include <QDir>
 #include <QFileDialog>
 #include <QMenu>
 #include <QMessageBox>
 #include <QTimeZone>
-#include <QUrl>
 
 #include "src/crypto/crypto_funcs.h"
 #include "src/gui/dlg_import_zfo.h"
@@ -40,6 +37,7 @@
 #include "src/io/filesystem.h"
 #include "src/io/isds_sessions.h"
 #include "src/log/log.h"
+#include "src/model_interaction/attachment_interaction.h"
 #include "src/settings/preferences.h"
 #include "src/views/table_home_end_filter.h"
 
@@ -49,6 +47,8 @@ DlgViewZfo::DlgViewZfo(const QString &zfoFileName, QWidget *parent)
     m_attachmentModel(this)
 {
 	setupUi(this);
+	/* Set default line height for table views/widgets. */
+	attachmentTable->setNarrowedLineHeight();
 
 	/* Load message ZFO. */
 	parseZfoFile(zfoFileName);
@@ -105,18 +105,18 @@ void DlgViewZfo::attachmentItemRightClicked(const QPoint &point)
 	QMenu *menu = new QMenu(this);
 
 	/* Detects selection of multiple attachments. */
-	QModelIndexList indexes = selectedAttachmentIndexes();
+	QModelIndexList indexes(
+	    AttachmentInteraction::selectedColumnIndexes(*attachmentTable,
+	        DbFlsTblModel::FNAME_COL));
 
 	if (index.isValid()) {
-		//attachmentItemSelectionChanged(index);
-
 		menu->addAction(QIcon(ICON_3PARTY_PATH "folder_16.png"),
 		    tr("Open attachment"), this,
 		    SLOT(openSelectedAttachment()))->
 		        setEnabled(indexes.size() == 1);
 		menu->addAction(QIcon(ICON_3PARTY_PATH "save_16.png"),
 		    tr("Save attachment"), this,
-		    SLOT(saveSelectedAttachmentToFile()))->
+		    SLOT(saveSelectedAttachmentsToFile()))->
 		        setEnabled(indexes.size() == 1);
 		menu->addAction(QIcon(ICON_3PARTY_PATH "save_16.png"),
 		    tr("Save attachments"), this,
@@ -128,159 +128,29 @@ void DlgViewZfo::attachmentItemRightClicked(const QPoint &point)
 	menu->exec(QCursor::pos());
 }
 
-void DlgViewZfo::saveSelectedAttachmentToFile(void)
+void DlgViewZfo::saveSelectedAttachmentsToFile(void)
 {
-	QModelIndex selectedIndex = selectedAttachmentIndex();
+	debugSlotCall();
 
-	if (!selectedIndex.isValid()) {
-		Q_ASSERT(0);
-		return;
-	}
-
-	QString fileName = selectedIndex.data().toString();
-	if (fileName.isEmpty()) {
-		Q_ASSERT(0);
-		return;
-	}
-	/* TODO -- Remember directory? */
-	fileName = QFileDialog::getSaveFileName(this,
-	    tr("Save attachment"), fileName);
-
-	if (fileName.isEmpty()) {
-		return;
-	}
-
-	QModelIndex dataIndex = selectedIndex.sibling(selectedIndex.row(),
-	    DbFlsTblModel::CONTENT_COL);
-	if (!dataIndex.isValid()) {
-		Q_ASSERT(0);
-		return;
-	}
-
-	QByteArray data =
-	    QByteArray::fromBase64(dataIndex.data().toByteArray());
-
-	if (WF_SUCCESS != writeFile(fileName, data)) {
-		QMessageBox::warning(this,
-		    tr("Error saving attachment."),
-		    tr("Cannot write file '%1'.").arg(fileName),
-		    QMessageBox::Ok);
-	}
+	AttachmentInteraction::saveAttachmentsToFile(this, *attachmentTable);
 }
 
 void DlgViewZfo::saveSelectedAttachmentsIntoDirectory(void)
 {
-	QModelIndexList selectedIndexes = selectedAttachmentIndexes();
+	debugSlotCall();
 
-	if (selectedIndexes.isEmpty()) {
-		Q_ASSERT(0);
-		return;
-	}
-
-	QString newDir = QFileDialog::getExistingDirectory(this,
-	    tr("Save attachments"), NULL,
-	    QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
-
-	if (newDir.isNull() || newDir.isEmpty()) {
-		return;
-	}
-	/* TODO -- Remember directory? */
-
-	bool unspecifiedFailed = false;
-	QList<QString> unsuccessfullFiles;
-
-	foreach (QModelIndex idx, selectedIndexes) {
-		if (!idx.isValid()) {
-			Q_ASSERT(0);
-			unspecifiedFailed = true;
-			continue;
-		}
-
-		QString fileName = idx.data().toString();
-		if (fileName.isEmpty()) {
-			Q_ASSERT(0);
-			unspecifiedFailed = true;
-			continue;
-		}
-
-		fileName = newDir + QDir::separator() + fileName;
-
-		QModelIndex dataIndex = idx.sibling(idx.row(),
-		    DbFlsTblModel::CONTENT_COL);
-		if (!dataIndex.isValid()) {
-			Q_ASSERT(0);
-			continue;
-		}
-
-		QByteArray data =
-		    QByteArray::fromBase64(dataIndex.data().toByteArray());
-
-		if (WF_SUCCESS != writeFile(fileName, data)) {
-			unsuccessfullFiles.append(fileName);
-			continue;
-		}
-	}
-
-	if (unspecifiedFailed) {
-		QMessageBox::warning(this,
-		    tr("Error saving attachments."),
-		    tr("Could not save all attachments."),
-		    QMessageBox::Ok);
-	} else if (!unsuccessfullFiles.isEmpty()) {
-		QString warnMsg =
-		    tr("In total %1 attachment files could not be written.")
-		        .arg(unsuccessfullFiles.size());
-		warnMsg += "\n" +
-		    tr("These are:").arg(unsuccessfullFiles.size()) + "\n";
-		int i;
-		for (i = 0; i < (unsuccessfullFiles.size() - 1); ++i) {
-			warnMsg += "    '" + unsuccessfullFiles.at(i) + "'\n";
-		}
-		warnMsg += "    '" + unsuccessfullFiles.at(i) + "'.";
-		QMessageBox::warning(this, tr("Error saving attachments."),
-		    warnMsg, QMessageBox::Ok);
-	}
+	AttachmentInteraction::saveAttachmentsToDirectory(this,
+	    *this->attachmentTable,
+	    AttachmentInteraction::selectedColumnIndexes(*attachmentTable,
+	        DbFlsTblModel::FNAME_COL));
 }
 
-void DlgViewZfo::openSelectedAttachment(void)
+void DlgViewZfo::openSelectedAttachment(const QModelIndex &index)
 {
-	QModelIndex selectedIndex = selectedAttachmentIndex();
+	debugSlotCall();
 
-	if (!selectedIndex.isValid()) {
-		Q_ASSERT(0);
-		return;
-	}
-
-	QString attachName = selectedIndex.data().toString();
-	if (attachName.isEmpty()) {
-		Q_ASSERT(0);
-		return;
-	}
-	attachName.replace(QRegExp("\\s"), "_").replace(
-	    QRegExp("[^a-zA-Z\\d\\.\\-_]"), "x");
-	/* TODO -- Add message id into file name? */
-	QString fileName = TMP_ATTACHMENT_PREFIX + attachName;
-
-	QModelIndex dataIndex = selectedIndex.sibling(selectedIndex.row(),
-	    DbFlsTblModel::CONTENT_COL);
-	if (!dataIndex.isValid()) {
-		Q_ASSERT(0);
-		return;
-	}
-
-	QByteArray data =
-	    QByteArray::fromBase64(dataIndex.data().toByteArray());
-
-	fileName = writeTemporaryFile(fileName, data);
-	if (!fileName.isEmpty()) {
-		QDesktopServices::openUrl(QUrl::fromLocalFile(fileName));
-		/* TODO -- Handle openUrl() return value. */
-	} else {
-		QMessageBox::warning(this,
-		    tr("Error opening attachment."),
-		    tr("Cannot write file '%1'.").arg(fileName),
-		    QMessageBox::Ok);
-	}
+	AttachmentInteraction::openAttachment(this, *this->attachmentTable,
+	    index);
 }
 
 void DlgViewZfo::showSignatureDetails(void)
@@ -368,6 +238,7 @@ void DlgViewZfo::setUpDialogue(void)
 		this->attachmentTable->setEnabled(true);
 		this->attachmentTable->show();
 		m_attachmentModel.setMessage(m_message);
+		m_attachmentModel.setHeader();
 		envelopeTextEdit->setHtml(
 		    messageDescriptionHtml(m_attachmentModel.rowCount(),
 		        m_message->raw, m_message->raw_length,
@@ -384,6 +255,10 @@ void DlgViewZfo::setUpDialogue(void)
 		    true);
 		attachmentTable->setColumnHidden(DbFlsTblModel::CONTENT_COL,
 		    true);
+		attachmentTable->setColumnHidden(DbFlsTblModel::MIME_COL,
+		    true);
+		attachmentTable->setColumnHidden(DbFlsTblModel::FPATH_COL,
+		    true);
 		attachmentTable->resizeColumnToContents(
 		    DbFlsTblModel::FNAME_COL);
 
@@ -391,7 +266,7 @@ void DlgViewZfo::setUpDialogue(void)
 		connect(attachmentTable, SIGNAL(customContextMenuRequested(QPoint)),
 		    this, SLOT(attachmentItemRightClicked(QPoint)));
 		connect(attachmentTable, SIGNAL(doubleClicked(QModelIndex)),
-		    this, SLOT(openSelectedAttachment()));
+		    this, SLOT(openSelectedAttachment(QModelIndex)));
 
 		attachmentTable->installEventFilter(new TableHomeEndFilter(this));
 	}
@@ -401,35 +276,6 @@ void DlgViewZfo::setUpDialogue(void)
 	    SLOT(showSignatureDetails()));
 }
 
-QModelIndex DlgViewZfo::selectedAttachmentIndex(void) const
-{
-	if (0 == attachmentTable->selectionModel()) {
-		Q_ASSERT(0);
-		return QModelIndex();
-	}
-
-	QModelIndex selectedIndex =
-	    attachmentTable->selectionModel()->currentIndex();
-
-	if (!selectedIndex.isValid()) {
-		Q_ASSERT(0);
-		return QModelIndex();
-	}
-
-	return selectedIndex.sibling(selectedIndex.row(),
-	    DbFlsTblModel::FNAME_COL);
-}
-
-QModelIndexList DlgViewZfo::selectedAttachmentIndexes(void) const
-{
-	if (0 == attachmentTable->selectionModel()) {
-		Q_ASSERT(0);
-		return QModelIndexList();
-	}
-
-	return attachmentTable->selectionModel()->selectedRows(0);
-}
-
 QString DlgViewZfo::messageDescriptionHtml(int attachmentCount,
     const void *msgDER, size_t msgSize, const void *tstDER,
     size_t tstSize) const
