@@ -90,6 +90,24 @@ public:
 		~MsgId(void)
 		{ }
 
+		MsgId &operator=(const MsgId &other) Q_DECL_NOTHROW
+		{
+			dmId = other.dmId;
+			deliveryTime = other.deliveryTime;
+			return *this;
+		}
+
+#ifdef Q_COMPILER_RVALUE_REFS
+		inline
+		MsgId &operator=(MsgId &&other) Q_DECL_NOTHROW
+		{
+			/* qSwap() is obsolete */
+			std::swap(dmId, other.dmId);
+			std::swap(deliveryTime, other.deliveryTime);
+			return *this;
+		}
+#endif /* Q_COMPILER_RVALUE_REFS */
+
 		bool isValid(void) const
 		{
 			return (dmId >= 0) && (deliveryTime.isValid());
@@ -110,6 +128,11 @@ public:
 		SoughtMsg(void)
 		    : mId(), type(0), dmAnnotation(), dmSender(), dmRecipient()
 		{ }
+		SoughtMsg(const SoughtMsg &msg)
+		    : mId(msg.mId), type(msg.type),
+		    dmAnnotation(msg.dmAnnotation), dmSender(msg.dmSender),
+		    dmRecipient(msg.dmRecipient)
+		{ }
 		SoughtMsg(const MsgId &id, int t, const QString &annot,
 		    const QString &sen, const QString &rec)
 		    : mId(id), type(t), dmAnnotation(annot),
@@ -124,6 +147,30 @@ public:
 		~SoughtMsg(void)
 		{ }
 
+		SoughtMsg &operator=(const SoughtMsg &other) Q_DECL_NOTHROW
+		{
+			mId = other.mId;
+			type = other.type;
+			dmAnnotation = other.dmAnnotation;
+			dmSender = other.dmSender;
+			dmRecipient = other.dmRecipient;
+			return *this;
+		}
+
+#ifdef Q_COMPILER_RVALUE_REFS
+		inline
+		SoughtMsg &operator=(SoughtMsg &&other) Q_DECL_NOTHROW
+		{
+			/* qSwap() is obsolete */
+			std::swap(mId, other.mId);
+			std::swap(type, other.type);
+			std::swap(dmAnnotation, other.dmAnnotation);
+			std::swap(dmSender, other.dmSender);
+			std::swap(dmRecipient, other.dmRecipient);
+			return *this;
+		}
+#endif /* Q_COMPILER_RVALUE_REFS */
+
 		bool isValid(void) const
 		{
 			return mId.isValid() &&
@@ -237,6 +284,14 @@ public:
 	 */
 	PartialEnvelopeData msgsReplyData(qint64 dmId) const;
 
+	/*!
+	 * @brief Return message type (sent or received).
+	 *
+	 * @param[in] dmId Message id.
+	 * @return Message type value, negative value on error.
+	 */
+	int msgMessageType(qint64 dmId) const;
+
 	/*!
 	 * @brief Returns true if verification attempt was performed.
 	 *
@@ -659,6 +714,14 @@ public:
 	bool copyRelevantMsgsToNewDb(const QString &newDbFileName,
 	   const QString &year);
 
+	/*!
+	 * @brief Get id of message in webdatovka db form ISDS msgId.
+	 *
+	 * @param[in] msgId  Message identifier.
+	 * @return Internal id of message in webdatovka db.
+	 */
+	int getWebDatokaId(qint64 msgId);
+
 protected: /* These function are used from within a database container. */
 	/*!
 	 * @brief Return all received messages model.
@@ -966,6 +1029,10 @@ protected: /* These function are used from within a database container. */
 
 	DbMsgsTblModel m_sqlMsgsModel; /*!< Model of displayed messages. */
 
+public:
+	static
+	const QVector<QString> fileItemIds;
+
 private:
 	static
 	const QVector<QString> msgPrintedAttribs;
@@ -973,8 +1040,6 @@ private:
 	const QVector<QString> msgDeliveryBoolAttribs;
 	static
 	const QVector<QString> msgStatus;
-	static
-	const QVector<QString> fileItemIds;
 
 	DbFlsTblModel m_sqlFilesModel; /*!< Model of displayed files. */
 
