@@ -29,7 +29,9 @@
 #include "src/io/tag_db.h" /* Direct access to tag database, */
 #include "src/io/db_tables.h"
 #include "src/io/dbs.h"
+#include "src/io/tag_db_container.h"
 #include "src/models/messages_model.h"
+#include "src/io/message_db.h"
 
 const int DbMsgsTblModel::rcvdMsgsColCnt(8);
 const int DbMsgsTblModel::sntMsgsColCnt(7);
@@ -530,7 +532,16 @@ DbMsgsTblModel &DbMsgsTblModel::dummyModel(enum DbMsgsTblModel::Type type)
 
 bool DbMsgsTblModel::fillTagsColumn(const QString &userName, int col)
 {
-	if (0 == globTagDbPtr) {
+	TagDb *tagDb = 0;
+
+	if (isWebDatovkaAccount(userName)) {
+		tagDb = globWebDatovkaTagDbPtr->
+		     accessTagDb(getWebDatovkaTagDbPrefix(userName));
+	} else {
+		tagDb = globTagDbPtr;
+	}
+
+	if (0 == tagDb) {
 		return false;
 	}
 
@@ -553,7 +564,7 @@ bool DbMsgsTblModel::fillTagsColumn(const QString &userName, int col)
 	for (int row = 0; row < rowCount(); ++row) {
 		qint64 dmId = TblModel::index(row, 0).data().toLongLong();
 		m_data[row][col] = QVariant::fromValue(
-		    globTagDbPtr->getMessageTags(userName, dmId));
+		    tagDb->getMessageTags(userName, dmId));
 	}
 
 	emit dataChanged(TblModel::index(0, col),
@@ -565,7 +576,16 @@ bool DbMsgsTblModel::fillTagsColumn(const QString &userName, int col)
 bool DbMsgsTblModel::refillTagsColumn(const QString &userName,
     const QList<qint64> &dmIds, int col)
 {
-	if (0 == globTagDbPtr) {
+	TagDb *tagDb = 0;
+
+	if (isWebDatovkaAccount(userName)) {
+		tagDb = globWebDatovkaTagDbPtr->
+		    accessTagDb(getWebDatovkaTagDbPrefix(userName));
+	} else {
+		tagDb = globTagDbPtr;
+	}
+
+	if (0 == tagDb) {
 		return false;
 	}
 
@@ -589,7 +609,7 @@ bool DbMsgsTblModel::refillTagsColumn(const QString &userName,
 		qint64 dmId = TblModel::index(row, 0).data().toLongLong();
 		if (dmIds.contains(dmId)) {
 			m_data[row][col] = QVariant::fromValue(
-			    globTagDbPtr->getMessageTags(userName, dmId));
+			    tagDb->getMessageTags(userName, dmId));
 			emit dataChanged(TblModel::index(row, col),
 			    TblModel::index(row, col));
 		}
