@@ -75,9 +75,11 @@ QVariant TblModel::headerData(int section, Qt::Orientation orientation,
 
 void TblModel::setQuery(QSqlQuery &query)
 {
+	beginResetModel();
 	m_data.clear();
 	m_rowsAllocated = 0;
 	m_rowCount = 0;
+	endResetModel();
 
 	/* Looks like empty results have column count set. */
 	m_columnCount = query.record().count();
@@ -96,16 +98,11 @@ bool TblModel::appendQueryData(QSqlQuery &query)
 	query.first();
 	while (query.isActive() && query.isValid()) {
 
-		if (m_rowCount == m_rowsAllocated) {
-			m_rowsAllocated += m_rowAllocationIncrement;
-			m_data.resize(m_rowsAllocated);
-		}
+		reserveSpace();
 
 		QVector<QVariant> row(m_columnCount);
 
-		for (int i = 0; i < m_columnCount; ++i) {
-			row[i] = query.value(i);
-		}
+		queryToVector(row, query);
 
 		m_data[m_rowCount++] = row;
 
@@ -117,6 +114,89 @@ bool TblModel::appendQueryData(QSqlQuery &query)
 	return true;
 }
 
+bool TblModel::moveRows(const QModelIndex &sourceParent, int sourceRow,
+    int count, const QModelIndex &destinationParent, int destinationChild)
+{
+	if (sourceParent.isValid() || destinationParent.isValid()) {
+		/* Only moves within root node are allowed. */
+		return false;
+	}
+
+	if ((sourceRow < 0) || (sourceRow >= rowCount()) ||
+	    (count < 0) || ((sourceRow + count) > rowCount()) ||
+	    (destinationChild < 0) || (destinationChild > rowCount())) {
+		/* Boundaries or location outside limits. */
+		return false;
+	}
+
+	if (count == 0) {
+		return true;
+	}
+
+	if ((sourceRow <= destinationChild) &&
+	    (destinationChild <= (sourceRow + count))) {
+		/* Destination lies within source. */
+		return true;
+	}
+
+	int newPosition; /* Position after removal. */
+	if (destinationChild < sourceRow) {
+		newPosition = destinationChild;
+	} else if (destinationChild > (sourceRow + count)) {
+		newPosition = destinationChild - count;
+	} else {
+		Q_ASSERT(0);
+		return false;
+	}
+
+	beginMoveRows(sourceParent, sourceRow, sourceRow + count - 1,
+	    destinationParent, destinationChild);
+
+	QList< QVector<QVariant> > movedData;
+	for (int i = sourceRow; i < (sourceRow + count); ++i) {
+		movedData.append(m_data.takeAt(sourceRow));
+	}
+
+	for (int i = 0; i < count; ++i) {
+		m_data.insert(newPosition + i, movedData.takeAt(0));
+	}
+	Q_ASSERT(movedData.isEmpty());
+
+	endMoveRows();
+
+	return true;
+}
+
+bool TblModel::removeRows(int row, int count, const QModelIndex &parent)
+{
+	if ((row < 0) || (count < 0) || parent.isValid()) {
+		/* Return if not operating on model root. */
+		return false;
+	}
+
+	if (row >= rowCount()) {
+		return false;
+	}
+
+	if (count > (rowCount() - row)) {
+		/* Adjust the count to fit the model. */
+		count = rowCount() - row;
+	}
+
+	int last = row + count - 1;
+	beginRemoveRows(parent, row, last);
+
+	/* Work in reverse order. */
+	for (int i = last; i >= row; --i) {
+		m_data.removeAt(i);
+		--m_rowCount;
+	}
+
+	endRemoveRows();
+
+	return true;
+}
+
 QVariant TblModel::_data(const QModelIndex &index, int role) const
 {
 	if (role != Qt::DisplayRole) {
@@ -150,3 +230,18 @@ QVariant TblModel::_headerData(int section, Qt::Orientation orientation,
 
 	return m_headerData[section][role];
 }
+
+void TblModel::reserveSpace(void)
+{
+	if (m_rowCount == m_rowsAllocated) {
+		m_rowsAllocated += m_rowAllocationIncrement;
+		m_data.resize(m_rowsAllocated);
+	}
+}
+
+void TblModel::queryToVector(QVector<QVariant> &vect, const QSqlQuery &query)
+{
+	for (int i = 0; i < query.record().count(); ++i) {
+		vect[i] = query.value(i);
+	}
+}
