@@ -30,6 +30,8 @@
 #include "src/common.h" // qdatovka_error, enum MessageDirection
 #include "src/io/message_db.h" // MessageDb::MsgId
 #include "src/io/message_db_set.h"
+#include "src/web/json.h"
+
 
 /*!
  * @brief Maximum length of message list to be downloaded.
@@ -46,6 +48,8 @@
 #define PL_IMPORT_ZFO_DINFO "ImportZfoDeliveryInfo"
 #define PL_IMPORT_ZFO_MSG "ImportZfoMessage"
 #define PL_SEND_MESSAGE "SendMessage"
+#define PL_SYNC_ACCOUNT "SyncAccount"
+#define PL_GET_ACCOUNT_LIST "GetAccounts"
 
 /*!
  * @brief This class contains generic functions that can be used in derived
@@ -89,6 +93,7 @@ public:
 	void run(void) = 0;
 
 protected:
+
 	/*!
 	 * @brief Store message delivery information into database.
 	 *
@@ -108,11 +113,13 @@ protected:
 	 * @param[in]     msgDirect Received or sent message.
 	 * @param[in,out] dbSet     Database container.
 	 * @param[in]     envel     Message envelope.
+	 * @param[in]     msgId     Message webdatovka id or NULL.
 	 * @return Error state.
 	 */
 	static
 	qdatovka_error storeEnvelope(enum MessageDirection msgDirect,
-	    MessageDbSet &dbSet, const struct isds_envelope *envel);
+	    MessageDbSet &dbSet, const struct isds_envelope *envel,
+	    QString msgId);
 
 	/*!
 	 * @brief Store message into database.
@@ -129,7 +136,7 @@ protected:
 	qdatovka_error storeMessage(bool signedMsg,
 	    enum MessageDirection msgDirect,
 	    MessageDbSet &dbSet, const struct isds_message *msg,
-	    const QString &progressLabel);
+	    const QString &progressLabel, QString msgId);
 
 	/*!
 	 * @brief Store attachments into database.
@@ -149,11 +156,26 @@ protected:
 	 * @param[in]     msgDirect Received or sent message.
 	 * @param[in,out] messageDb Database.
 	 * @param[in]     envel     Message envelope.
+	 * @param[in]     msgId     Message webdatovka id or NULL.
 	 * @return True on success.
 	 */
 	static
 	qdatovka_error updateEnvelope(enum MessageDirection msgDirect,
-	    MessageDb &messageDb, const struct isds_envelope *envel);
+	    MessageDb &messageDb, const struct isds_envelope *envel,
+	    QString msgId);
+
+	/*!
+	 * @brief Store envelope into database.
+	 *
+	 * @param[in]     msgDirect Received or sent message.
+	 * @param[in,out] dbSet     Database container.
+	 * @param[in]     envel     Message envelope.
+	 * @param[in]     isNew     true = insert into db, false = update in db.
+	 * @return Error state.
+	 */
+	static
+	qdatovka_error storeEnvelopeWebDatovka(enum MessageDirection msgDirect,
+	    MessageDbSet &dbSet, const JsonLayer::Envelope &envel, bool isNew);
 
 private:
 //	/*!
