@@ -33,19 +33,18 @@
 #include "src/worker/task_verify_message.h"
 
 TaskVerifyMessage::TaskVerifyMessage(const QString &userName,
-    MessageDbSet *dbSet, qint64 dmId, const QDateTime &deliveryTime)
+    MessageDbSet *dbSet, const MessageDb::MsgId &msgId)
     : m_result(VERIFY_ERR),
     m_isdsError(),
     m_isdsLongError(),
     m_userName(userName),
     m_dbSet(dbSet),
-    m_dmId(dmId),
-    m_deliveryTime(deliveryTime)
+    m_msgId(msgId)
 {
 	Q_ASSERT(!m_userName.isEmpty());
 	Q_ASSERT(0 != m_dbSet);
-	Q_ASSERT(m_dmId >= 0);
-	Q_ASSERT(m_deliveryTime.isValid());
+	Q_ASSERT(m_msgId.dmId >= 0);
+	Q_ASSERT(m_msgId.deliveryTime.isValid());
 }
 
 void TaskVerifyMessage::run(void)
@@ -60,12 +59,12 @@ void TaskVerifyMessage::run(void)
 		return;
 	}
 
-	if (0 > m_dmId) {
+	if (0 > m_msgId.dmId) {
 		Q_ASSERT(0);
 		return;
 	}
 
-	if (!m_deliveryTime.isValid()) {
+	if (!m_msgId.deliveryTime.isValid()) {
 		Q_ASSERT(0);
 		return;
 	}
@@ -75,8 +74,8 @@ void TaskVerifyMessage::run(void)
 
 	/* ### Worker task begin. ### */
 
-	m_result = verifyMessage(m_userName, m_dbSet, m_dmId, m_deliveryTime,
-	    m_isdsError, m_isdsLongError);
+	m_result = verifyMessage(m_userName, m_dbSet, m_msgId, m_isdsError,
+	    m_isdsLongError);
 
 	emit globMsgProcEmitter.progressChange(PL_IDLE, 0);
 
@@ -128,15 +127,16 @@ struct isds_hash *localMessageHash(MessageDb *messageDb, qint64 dmId)
 }
 
 enum TaskVerifyMessage::Result TaskVerifyMessage::verifyMessage(
-    const QString &userName, MessageDbSet *dbSet, qint64 dmId,
-    const QDateTime &deliveryTime, QString &error, QString &longError)
+    const QString &userName, MessageDbSet *dbSet, const MessageDb::MsgId &msgId,
+    QString &error, QString &longError)
 {
 	Q_ASSERT(!userName.isEmpty());
 	Q_ASSERT(0 != dbSet);
-	Q_ASSERT(dmId >= 0);
-	Q_ASSERT(deliveryTime.isValid());
+	Q_ASSERT(msgId.dmId >= 0);
+	Q_ASSERT(msgId.deliveryTime.isValid());
 
-	MessageDb *messageDb = dbSet->accessMessageDb(deliveryTime, false);
+	MessageDb *messageDb = dbSet->accessMessageDb(msgId.deliveryTime,
+	    false);
 	if (0 == messageDb) {
 		Q_ASSERT(0);
 		return VERIFY_ERR;
@@ -150,10 +150,10 @@ enum TaskVerifyMessage::Result TaskVerifyMessage::verifyMessage(
 
 	struct isds_hash *hashIsds = NULL;
 	isds_error status = isds_download_message_hash(session,
-	    QString::number(dmId).toUtf8().constData(), &hashIsds);
+	    QString::number(msgId.dmId).toUtf8().constData(), &hashIsds);
 	if (IE_SUCCESS != status) {
 		logErrorNL("Error downloading hash of message '%" PRId64 "'.",
-		    dmId);
+		    msgId.dmId);
 		error = isds_error(status);
 		longError = isdsLongMessage(session);
 		isds_hash_free(&hashIsds);
@@ -162,11 +162,11 @@ enum TaskVerifyMessage::Result TaskVerifyMessage::verifyMessage(
 
 	Q_ASSERT(NULL != hashIsds);
 
-	struct isds_hash *hashLocal = localMessageHash(messageDb, dmId);
+	struct isds_hash *hashLocal = localMessageHash(messageDb, msgId.dmId);
 	if (NULL == hashLocal) {
 		logErrorNL(
 		    "Error obtaining hash of message '%" PRId64 "' from local database.",
-		    dmId);
+		    msgId.dmId);
 		isds_hash_free(&hashIsds);
 		return VERIFY_SQL_ERR;
 	}
